/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * This file was generated.
 *  with google-apis-code-generator 1.3.0 (build: 2013-05-06 19:17:05 UTC)
 *  on 2013-05-07 at 00:01:23 UTC 
 */

package com.google.api.services.licensing;

import com.google.api.client.googleapis.GoogleUtils;
import com.google.api.client.googleapis.services.AbstractGoogleClientRequest;
import com.google.api.client.googleapis.services.GoogleClientRequestInitializer;
import com.google.api.client.googleapis.services.json.AbstractGoogleJsonClient;
import com.google.api.client.http.HttpRequestInitializer;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.JsonObjectParser;
import com.google.common.base.Preconditions;

/**
 * Service definition for Licensing (v1).
 *
 * <p>
 * Licensing API to view and manage license for your domain.
 * </p>
 *
 * <p>
 * For more information about this service, see the
 * <a href="https://developers.google.com/google-apps/licensing/" target="_blank">API Documentation</a>
 * </p>
 *
 * <p>
 * This service uses {@link LicensingRequestInitializer} to initialize global parameters via its
 * {@link Builder}.
 * </p>
 *
 * <p>
 * Upgrade warning: this class now extends {@link AbstractGoogleJsonClient}, whereas in prior
 * version 1.8 it extended {@link com.google.api.client.googleapis.services.GoogleClient}.
 * </p>
 *
 * @since 1.3
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public class Licensing extends AbstractGoogleJsonClient {

  // Note: Leave this static initializer at the top of the file.
  static {
    Preconditions.checkState(GoogleUtils.VERSION.equals("1.13.2-beta"),
        "You are currently running with version %s of google-api-client. " +
        "You need version 1.13.2-beta of google-api-client to run version " +
        "1.13.2-beta of the Enterprise License Manager API library.", GoogleUtils.VERSION);
  }

  /**
   * The default encoded root URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_ROOT_URL = "https://www.googleapis.com/";

  /**
   * The default encoded service path of the service. This is determined when the library is
   * generated and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_SERVICE_PATH = "apps/licensing/v1/product/";

  /**
   * The default encoded base URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   * @deprecated (scheduled to be removed in 1.13)
   */
  @Deprecated
  public static final String DEFAULT_BASE_URL = DEFAULT_ROOT_URL + DEFAULT_SERVICE_PATH;

  /**
   * Constructor.
   *
   * <p>
   * Use {@link Builder} if you need to specify any of the optional parameters.
   * </p>
   *
   * @param transport HTTP transport
   * @param jsonFactory JSON factory
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @since 1.7
   */
  public Licensing(HttpTransport transport, JsonFactory jsonFactory,
      HttpRequestInitializer httpRequestInitializer) {
    super(transport,
        jsonFactory,
        DEFAULT_ROOT_URL,
        DEFAULT_SERVICE_PATH,
        httpRequestInitializer,
        false);
  }

  /**
   * @param transport HTTP transport
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @param rootUrl root URL of the service
   * @param servicePath service path
   * @param jsonObjectParser JSON object parser
   * @param googleClientRequestInitializer Google request initializer or {@code null} for none
   * @param applicationName application name to be sent in the User-Agent header of requests or
   *        {@code null} for none
   * @param suppressPatternChecks whether discovery pattern checks should be suppressed on required
   *        parameters
   */
  Licensing(HttpTransport transport,
      HttpRequestInitializer httpRequestInitializer,
      String rootUrl,
      String servicePath,
      JsonObjectParser jsonObjectParser,
      GoogleClientRequestInitializer googleClientRequestInitializer,
      String applicationName,
      boolean suppressPatternChecks) {
    super(transport,
        httpRequestInitializer,
        rootUrl,
        servicePath,
        jsonObjectParser,
        googleClientRequestInitializer,
        applicationName,
        suppressPatternChecks);
  }

  @Override
  protected void initialize(AbstractGoogleClientRequest<?> httpClientRequest) throws java.io.IOException {
    super.initialize(httpClientRequest);
  }

  /**
   * An accessor for creating requests from the LicenseAssignments collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Licensing licensing = new Licensing(...);}
   *   {@code Licensing.LicenseAssignments.List request = licensing.licenseAssignments().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public LicenseAssignments licenseAssignments() {
    return new LicenseAssignments();
  }

  /**
   * The "licenseAssignments" collection of methods.
   */
  public class LicenseAssignments {

    /**
     * Revoke License.
     *
     * Create a request for the method "licenseAssignments.delete".
     *
     * This request holds the parameters needed by the the licensing server.  After setting any optional
     * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
     *
     * @param productId Name for product
     * @param skuId Name for sku
     * @param userId email id or unique Id of the user
     * @return the request
     */
    public Delete delete(String productId, String skuId, String userId) throws java.io.IOException {
      Delete result = new Delete(productId, skuId, userId);
      initialize(result);
      return result;
    }

    public class Delete extends LicensingRequest<Void> {

      private static final String REST_PATH = "{productId}/sku/{skuId}/user/{userId}";

      /**
       * Revoke License.
       *
       * Create a request for the method "licenseAssignments.delete".
       *
       * This request holds the parameters needed by the the licensing server.  After setting any
       * optional parameters, call the {@link Delete#execute()} method to invoke the remote operation.
       * <p> {@link Delete#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param productId Name for product
       * @param skuId Name for sku
       * @param userId email id or unique Id of the user
       * @since 1.13
       */
      protected Delete(String productId, String skuId, String userId) {
        super(Licensing.this, "DELETE", REST_PATH, null, Void.class);
        this.productId = Preconditions.checkNotNull(productId, "Required parameter productId must be specified.");
        this.skuId = Preconditions.checkNotNull(skuId, "Required parameter skuId must be specified.");
        this.userId = Preconditions.checkNotNull(userId, "Required parameter userId must be specified.");
      }

      @Override
      public Delete setAlt(String alt) {
        return (Delete) super.setAlt(alt);
      }

      @Override
      public Delete setFields(String fields) {
        return (Delete) super.setFields(fields);
      }

      @Override
      public Delete setKey(String key) {
        return (Delete) super.setKey(key);
      }

      @Override
      public Delete setOauthToken(String oauthToken) {
        return (Delete) super.setOauthToken(oauthToken);
      }

      @Override
      public Delete setPrettyPrint(Boolean prettyPrint) {
        return (Delete) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Delete setQuotaUser(String quotaUser) {
        return (Delete) super.setQuotaUser(quotaUser);
      }

      @Override
      public Delete setUserIp(String userIp) {
        return (Delete) super.setUserIp(userIp);
      }

      /** Name for product */
      @com.google.api.client.util.Key
      private String productId;

      /** Name for product
       */
      public String getProductId() {
        return productId;
      }

      /** Name for product */
      public Delete setProductId(String productId) {
        this.productId = productId;
        return this;
      }

      /** Name for sku */
      @com.google.api.client.util.Key
      private String skuId;

      /** Name for sku
       */
      public String getSkuId() {
        return skuId;
      }

      /** Name for sku */
      public Delete setSkuId(String skuId) {
        this.skuId = skuId;
        return this;
      }

      /** email id or unique Id of the user */
      @com.google.api.client.util.Key
      private String userId;

      /** email id or unique Id of the user
       */
      public String getUserId() {
        return userId;
      }

      /** email id or unique Id of the user */
      public Delete setUserId(String userId) {
        this.userId = userId;
        return this;
      }

    }
    /**
     * Get license assignment of a particular product and sku for a user
     *
     * Create a request for the method "licenseAssignments.get".
     *
     * This request holds the parameters needed by the the licensing server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param productId Name for product
     * @param skuId Name for sku
     * @param userId email id or unique Id of the user
     * @return the request
     */
    public Get get(String productId, String skuId, String userId) throws java.io.IOException {
      Get result = new Get(productId, skuId, userId);
      initialize(result);
      return result;
    }

    public class Get extends LicensingRequest<com.google.api.services.licensing.model.LicenseAssignment> {

      private static final String REST_PATH = "{productId}/sku/{skuId}/user/{userId}";

      /**
       * Get license assignment of a particular product and sku for a user
       *
       * Create a request for the method "licenseAssignments.get".
       *
       * This request holds the parameters needed by the the licensing server.  After setting any
       * optional parameters, call the {@link Get#execute()} method to invoke the remote operation. <p>
       * {@link Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param productId Name for product
       * @param skuId Name for sku
       * @param userId email id or unique Id of the user
       * @since 1.13
       */
      protected Get(String productId, String skuId, String userId) {
        super(Licensing.this, "GET", REST_PATH, null, com.google.api.services.licensing.model.LicenseAssignment.class);
        this.productId = Preconditions.checkNotNull(productId, "Required parameter productId must be specified.");
        this.skuId = Preconditions.checkNotNull(skuId, "Required parameter skuId must be specified.");
        this.userId = Preconditions.checkNotNull(userId, "Required parameter userId must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** Name for product */
      @com.google.api.client.util.Key
      private String productId;

      /** Name for product
       */
      public String getProductId() {
        return productId;
      }

      /** Name for product */
      public Get setProductId(String productId) {
        this.productId = productId;
        return this;
      }

      /** Name for sku */
      @com.google.api.client.util.Key
      private String skuId;

      /** Name for sku
       */
      public String getSkuId() {
        return skuId;
      }

      /** Name for sku */
      public Get setSkuId(String skuId) {
        this.skuId = skuId;
        return this;
      }

      /** email id or unique Id of the user */
      @com.google.api.client.util.Key
      private String userId;

      /** email id or unique Id of the user
       */
      public String getUserId() {
        return userId;
      }

      /** email id or unique Id of the user */
      public Get setUserId(String userId) {
        this.userId = userId;
        return this;
      }

    }
    /**
     * Assign License.
     *
     * Create a request for the method "licenseAssignments.insert".
     *
     * This request holds the parameters needed by the the licensing server.  After setting any optional
     * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
     *
     * @param productId Name for product
     * @param skuId Name for sku
     * @param content the {@link com.google.api.services.licensing.model.LicenseAssignmentInsert}
     * @return the request
     */
    public Insert insert(String productId, String skuId, com.google.api.services.licensing.model.LicenseAssignmentInsert content) throws java.io.IOException {
      Insert result = new Insert(productId, skuId, content);
      initialize(result);
      return result;
    }

    public class Insert extends LicensingRequest<com.google.api.services.licensing.model.LicenseAssignment> {

      private static final String REST_PATH = "{productId}/sku/{skuId}/user";

      /**
       * Assign License.
       *
       * Create a request for the method "licenseAssignments.insert".
       *
       * This request holds the parameters needed by the the licensing server.  After setting any
       * optional parameters, call the {@link Insert#execute()} method to invoke the remote operation.
       * <p> {@link Insert#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param productId Name for product
       * @param skuId Name for sku
       * @param content the {@link com.google.api.services.licensing.model.LicenseAssignmentInsert}
       * @since 1.13
       */
      protected Insert(String productId, String skuId, com.google.api.services.licensing.model.LicenseAssignmentInsert content) {
        super(Licensing.this, "POST", REST_PATH, content, com.google.api.services.licensing.model.LicenseAssignment.class);
        this.productId = Preconditions.checkNotNull(productId, "Required parameter productId must be specified.");
        this.skuId = Preconditions.checkNotNull(skuId, "Required parameter skuId must be specified.");
      }

      @Override
      public Insert setAlt(String alt) {
        return (Insert) super.setAlt(alt);
      }

      @Override
      public Insert setFields(String fields) {
        return (Insert) super.setFields(fields);
      }

      @Override
      public Insert setKey(String key) {
        return (Insert) super.setKey(key);
      }

      @Override
      public Insert setOauthToken(String oauthToken) {
        return (Insert) super.setOauthToken(oauthToken);
      }

      @Override
      public Insert setPrettyPrint(Boolean prettyPrint) {
        return (Insert) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Insert setQuotaUser(String quotaUser) {
        return (Insert) super.setQuotaUser(quotaUser);
      }

      @Override
      public Insert setUserIp(String userIp) {
        return (Insert) super.setUserIp(userIp);
      }

      /** Name for product */
      @com.google.api.client.util.Key
      private String productId;

      /** Name for product
       */
      public String getProductId() {
        return productId;
      }

      /** Name for product */
      public Insert setProductId(String productId) {
        this.productId = productId;
        return this;
      }

      /** Name for sku */
      @com.google.api.client.util.Key
      private String skuId;

      /** Name for sku
       */
      public String getSkuId() {
        return skuId;
      }

      /** Name for sku */
      public Insert setSkuId(String skuId) {
        this.skuId = skuId;
        return this;
      }

    }
    /**
     * List license assignments for given product of the customer.
     *
     * Create a request for the method "licenseAssignments.listForProduct".
     *
     * This request holds the parameters needed by the the licensing server.  After setting any optional
     * parameters, call the {@link ListForProduct#execute()} method to invoke the remote operation.
     *
     * @param productId Name for product
     * @param customerId CustomerId represents the customer for whom licenseassignments are queried
     * @return the request
     */
    public ListForProduct listForProduct(String productId, String customerId) throws java.io.IOException {
      ListForProduct result = new ListForProduct(productId, customerId);
      initialize(result);
      return result;
    }

    public class ListForProduct extends LicensingRequest<com.google.api.services.licensing.model.LicenseAssignmentList> {

      private static final String REST_PATH = "{productId}/users";

      /**
       * List license assignments for given product of the customer.
       *
       * Create a request for the method "licenseAssignments.listForProduct".
       *
       * This request holds the parameters needed by the the licensing server.  After setting any
       * optional parameters, call the {@link ListForProduct#execute()} method to invoke the remote
       * operation. <p> {@link ListForProduct#initialize(AbstractGoogleClientRequest)} must be called to
       * initialize this instance immediately after invoking the constructor. </p>
       *
       * @param productId Name for product
       * @param customerId CustomerId represents the customer for whom licenseassignments are queried
       * @since 1.13
       */
      protected ListForProduct(String productId, String customerId) {
        super(Licensing.this, "GET", REST_PATH, null, com.google.api.services.licensing.model.LicenseAssignmentList.class);
        this.productId = Preconditions.checkNotNull(productId, "Required parameter productId must be specified.");
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public ListForProduct setAlt(String alt) {
        return (ListForProduct) super.setAlt(alt);
      }

      @Override
      public ListForProduct setFields(String fields) {
        return (ListForProduct) super.setFields(fields);
      }

      @Override
      public ListForProduct setKey(String key) {
        return (ListForProduct) super.setKey(key);
      }

      @Override
      public ListForProduct setOauthToken(String oauthToken) {
        return (ListForProduct) super.setOauthToken(oauthToken);
      }

      @Override
      public ListForProduct setPrettyPrint(Boolean prettyPrint) {
        return (ListForProduct) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public ListForProduct setQuotaUser(String quotaUser) {
        return (ListForProduct) super.setQuotaUser(quotaUser);
      }

      @Override
      public ListForProduct setUserIp(String userIp) {
        return (ListForProduct) super.setUserIp(userIp);
      }

      /** Name for product */
      @com.google.api.client.util.Key
      private String productId;

      /** Name for product
       */
      public String getProductId() {
        return productId;
      }

      /** Name for product */
      public ListForProduct setProductId(String productId) {
        this.productId = productId;
        return this;
      }

      /** CustomerId represents the customer for whom licenseassignments are queried */
      @com.google.api.client.util.Key
      private String customerId;

      /** CustomerId represents the customer for whom licenseassignments are queried
       */
      public String getCustomerId() {
        return customerId;
      }

      /** CustomerId represents the customer for whom licenseassignments are queried */
      public ListForProduct setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Token to fetch the next page.Optional. By default server will return first page */
      @com.google.api.client.util.Key
      private String pageToken;

      /** Token to fetch the next page.Optional. By default server will return first page
       */
      public String getPageToken() {
        return pageToken;
      }

      /** Token to fetch the next page.Optional. By default server will return first page */
      public ListForProduct setPageToken(String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /**
     * Maximum number of campaigns to return at one time. Must be positive. Optional. Default value
     * is 100.
     */
      @com.google.api.client.util.Key
      private Long maxResults;

      /** Maximum number of campaigns to return at one time. Must be positive. Optional. Default value is
     100. [default: 100] [minimum: 1] [maximum: 1000]
       */
      public Long getMaxResults() {
        return maxResults;
      }

      /**
     * Maximum number of campaigns to return at one time. Must be positive. Optional. Default value
     * is 100.
     */
      public ListForProduct setMaxResults(Long maxResults) {
        this.maxResults = maxResults;
        return this;
      }

    }
    /**
     * List license assignments for given product and sku of the customer.
     *
     * Create a request for the method "licenseAssignments.listForProductAndSku".
     *
     * This request holds the parameters needed by the the licensing server.  After setting any optional
     * parameters, call the {@link ListForProductAndSku#execute()} method to invoke the remote
     * operation.
     *
     * @param productId Name for product
     * @param skuId Name for sku
     * @param customerId CustomerId represents the customer for whom licenseassignments are queried
     * @return the request
     */
    public ListForProductAndSku listForProductAndSku(String productId, String skuId, String customerId) throws java.io.IOException {
      ListForProductAndSku result = new ListForProductAndSku(productId, skuId, customerId);
      initialize(result);
      return result;
    }

    public class ListForProductAndSku extends LicensingRequest<com.google.api.services.licensing.model.LicenseAssignmentList> {

      private static final String REST_PATH = "{productId}/sku/{skuId}/users";

      /**
       * List license assignments for given product and sku of the customer.
       *
       * Create a request for the method "licenseAssignments.listForProductAndSku".
       *
       * This request holds the parameters needed by the the licensing server.  After setting any
       * optional parameters, call the {@link ListForProductAndSku#execute()} method to invoke the
       * remote operation. <p> {@link ListForProductAndSku#initialize(AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param productId Name for product
       * @param skuId Name for sku
       * @param customerId CustomerId represents the customer for whom licenseassignments are queried
       * @since 1.13
       */
      protected ListForProductAndSku(String productId, String skuId, String customerId) {
        super(Licensing.this, "GET", REST_PATH, null, com.google.api.services.licensing.model.LicenseAssignmentList.class);
        this.productId = Preconditions.checkNotNull(productId, "Required parameter productId must be specified.");
        this.skuId = Preconditions.checkNotNull(skuId, "Required parameter skuId must be specified.");
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public ListForProductAndSku setAlt(String alt) {
        return (ListForProductAndSku) super.setAlt(alt);
      }

      @Override
      public ListForProductAndSku setFields(String fields) {
        return (ListForProductAndSku) super.setFields(fields);
      }

      @Override
      public ListForProductAndSku setKey(String key) {
        return (ListForProductAndSku) super.setKey(key);
      }

      @Override
      public ListForProductAndSku setOauthToken(String oauthToken) {
        return (ListForProductAndSku) super.setOauthToken(oauthToken);
      }

      @Override
      public ListForProductAndSku setPrettyPrint(Boolean prettyPrint) {
        return (ListForProductAndSku) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public ListForProductAndSku setQuotaUser(String quotaUser) {
        return (ListForProductAndSku) super.setQuotaUser(quotaUser);
      }

      @Override
      public ListForProductAndSku setUserIp(String userIp) {
        return (ListForProductAndSku) super.setUserIp(userIp);
      }

      /** Name for product */
      @com.google.api.client.util.Key
      private String productId;

      /** Name for product
       */
      public String getProductId() {
        return productId;
      }

      /** Name for product */
      public ListForProductAndSku setProductId(String productId) {
        this.productId = productId;
        return this;
      }

      /** Name for sku */
      @com.google.api.client.util.Key
      private String skuId;

      /** Name for sku
       */
      public String getSkuId() {
        return skuId;
      }

      /** Name for sku */
      public ListForProductAndSku setSkuId(String skuId) {
        this.skuId = skuId;
        return this;
      }

      /** CustomerId represents the customer for whom licenseassignments are queried */
      @com.google.api.client.util.Key
      private String customerId;

      /** CustomerId represents the customer for whom licenseassignments are queried
       */
      public String getCustomerId() {
        return customerId;
      }

      /** CustomerId represents the customer for whom licenseassignments are queried */
      public ListForProductAndSku setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Token to fetch the next page.Optional. By default server will return first page */
      @com.google.api.client.util.Key
      private String pageToken;

      /** Token to fetch the next page.Optional. By default server will return first page
       */
      public String getPageToken() {
        return pageToken;
      }

      /** Token to fetch the next page.Optional. By default server will return first page */
      public ListForProductAndSku setPageToken(String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /**
     * Maximum number of campaigns to return at one time. Must be positive. Optional. Default value
     * is 100.
     */
      @com.google.api.client.util.Key
      private Long maxResults;

      /** Maximum number of campaigns to return at one time. Must be positive. Optional. Default value is
     100. [default: 100] [minimum: 1] [maximum: 1000]
       */
      public Long getMaxResults() {
        return maxResults;
      }

      /**
     * Maximum number of campaigns to return at one time. Must be positive. Optional. Default value
     * is 100.
     */
      public ListForProductAndSku setMaxResults(Long maxResults) {
        this.maxResults = maxResults;
        return this;
      }

    }
    /**
     * Assign License. This method supports patch semantics.
     *
     * Create a request for the method "licenseAssignments.patch".
     *
     * This request holds the parameters needed by the the licensing server.  After setting any optional
     * parameters, call the {@link Patch#execute()} method to invoke the remote operation.
     *
     * @param productId Name for product
     * @param skuId Name for sku for which license would be revoked
     * @param userId email id or unique Id of the user
     * @param content the {@link com.google.api.services.licensing.model.LicenseAssignment}
     * @return the request
     */
    public Patch patch(String productId, String skuId, String userId, com.google.api.services.licensing.model.LicenseAssignment content) throws java.io.IOException {
      Patch result = new Patch(productId, skuId, userId, content);
      initialize(result);
      return result;
    }

    public class Patch extends LicensingRequest<com.google.api.services.licensing.model.LicenseAssignment> {

      private static final String REST_PATH = "{productId}/sku/{skuId}/user/{userId}";

      /**
       * Assign License. This method supports patch semantics.
       *
       * Create a request for the method "licenseAssignments.patch".
       *
       * This request holds the parameters needed by the the licensing server.  After setting any
       * optional parameters, call the {@link Patch#execute()} method to invoke the remote operation.
       * <p> {@link Patch#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param productId Name for product
       * @param skuId Name for sku for which license would be revoked
       * @param userId email id or unique Id of the user
       * @param content the {@link com.google.api.services.licensing.model.LicenseAssignment}
       * @since 1.13
       */
      protected Patch(String productId, String skuId, String userId, com.google.api.services.licensing.model.LicenseAssignment content) {
        super(Licensing.this, "PATCH", REST_PATH, content, com.google.api.services.licensing.model.LicenseAssignment.class);
        this.productId = Preconditions.checkNotNull(productId, "Required parameter productId must be specified.");
        this.skuId = Preconditions.checkNotNull(skuId, "Required parameter skuId must be specified.");
        this.userId = Preconditions.checkNotNull(userId, "Required parameter userId must be specified.");
      }

      @Override
      public Patch setAlt(String alt) {
        return (Patch) super.setAlt(alt);
      }

      @Override
      public Patch setFields(String fields) {
        return (Patch) super.setFields(fields);
      }

      @Override
      public Patch setKey(String key) {
        return (Patch) super.setKey(key);
      }

      @Override
      public Patch setOauthToken(String oauthToken) {
        return (Patch) super.setOauthToken(oauthToken);
      }

      @Override
      public Patch setPrettyPrint(Boolean prettyPrint) {
        return (Patch) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Patch setQuotaUser(String quotaUser) {
        return (Patch) super.setQuotaUser(quotaUser);
      }

      @Override
      public Patch setUserIp(String userIp) {
        return (Patch) super.setUserIp(userIp);
      }

      /** Name for product */
      @com.google.api.client.util.Key
      private String productId;

      /** Name for product
       */
      public String getProductId() {
        return productId;
      }

      /** Name for product */
      public Patch setProductId(String productId) {
        this.productId = productId;
        return this;
      }

      /** Name for sku for which license would be revoked */
      @com.google.api.client.util.Key
      private String skuId;

      /** Name for sku for which license would be revoked
       */
      public String getSkuId() {
        return skuId;
      }

      /** Name for sku for which license would be revoked */
      public Patch setSkuId(String skuId) {
        this.skuId = skuId;
        return this;
      }

      /** email id or unique Id of the user */
      @com.google.api.client.util.Key
      private String userId;

      /** email id or unique Id of the user
       */
      public String getUserId() {
        return userId;
      }

      /** email id or unique Id of the user */
      public Patch setUserId(String userId) {
        this.userId = userId;
        return this;
      }

    }
    /**
     * Assign License.
     *
     * Create a request for the method "licenseAssignments.update".
     *
     * This request holds the parameters needed by the the licensing server.  After setting any optional
     * parameters, call the {@link Update#execute()} method to invoke the remote operation.
     *
     * @param productId Name for product
     * @param skuId Name for sku for which license would be revoked
     * @param userId email id or unique Id of the user
     * @param content the {@link com.google.api.services.licensing.model.LicenseAssignment}
     * @return the request
     */
    public Update update(String productId, String skuId, String userId, com.google.api.services.licensing.model.LicenseAssignment content) throws java.io.IOException {
      Update result = new Update(productId, skuId, userId, content);
      initialize(result);
      return result;
    }

    public class Update extends LicensingRequest<com.google.api.services.licensing.model.LicenseAssignment> {

      private static final String REST_PATH = "{productId}/sku/{skuId}/user/{userId}";

      /**
       * Assign License.
       *
       * Create a request for the method "licenseAssignments.update".
       *
       * This request holds the parameters needed by the the licensing server.  After setting any
       * optional parameters, call the {@link Update#execute()} method to invoke the remote operation.
       * <p> {@link Update#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param productId Name for product
       * @param skuId Name for sku for which license would be revoked
       * @param userId email id or unique Id of the user
       * @param content the {@link com.google.api.services.licensing.model.LicenseAssignment}
       * @since 1.13
       */
      protected Update(String productId, String skuId, String userId, com.google.api.services.licensing.model.LicenseAssignment content) {
        super(Licensing.this, "PUT", REST_PATH, content, com.google.api.services.licensing.model.LicenseAssignment.class);
        this.productId = Preconditions.checkNotNull(productId, "Required parameter productId must be specified.");
        this.skuId = Preconditions.checkNotNull(skuId, "Required parameter skuId must be specified.");
        this.userId = Preconditions.checkNotNull(userId, "Required parameter userId must be specified.");
      }

      @Override
      public Update setAlt(String alt) {
        return (Update) super.setAlt(alt);
      }

      @Override
      public Update setFields(String fields) {
        return (Update) super.setFields(fields);
      }

      @Override
      public Update setKey(String key) {
        return (Update) super.setKey(key);
      }

      @Override
      public Update setOauthToken(String oauthToken) {
        return (Update) super.setOauthToken(oauthToken);
      }

      @Override
      public Update setPrettyPrint(Boolean prettyPrint) {
        return (Update) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Update setQuotaUser(String quotaUser) {
        return (Update) super.setQuotaUser(quotaUser);
      }

      @Override
      public Update setUserIp(String userIp) {
        return (Update) super.setUserIp(userIp);
      }

      /** Name for product */
      @com.google.api.client.util.Key
      private String productId;

      /** Name for product
       */
      public String getProductId() {
        return productId;
      }

      /** Name for product */
      public Update setProductId(String productId) {
        this.productId = productId;
        return this;
      }

      /** Name for sku for which license would be revoked */
      @com.google.api.client.util.Key
      private String skuId;

      /** Name for sku for which license would be revoked
       */
      public String getSkuId() {
        return skuId;
      }

      /** Name for sku for which license would be revoked */
      public Update setSkuId(String skuId) {
        this.skuId = skuId;
        return this;
      }

      /** email id or unique Id of the user */
      @com.google.api.client.util.Key
      private String userId;

      /** email id or unique Id of the user
       */
      public String getUserId() {
        return userId;
      }

      /** email id or unique Id of the user */
      public Update setUserId(String userId) {
        this.userId = userId;
        return this;
      }

    }

  }

  /**
   * Builder for {@link Licensing}.
   *
   * <p>
   * Implementation is not thread-safe.
   * </p>
   *
   * @since 1.3.0
   */
  public static final class Builder extends AbstractGoogleJsonClient.Builder {

    /**
     * Returns an instance of a new builder.
     *
     * @param transport HTTP transport
     * @param jsonFactory JSON factory
     * @param httpRequestInitializer HTTP request initializer or {@code null} for none
     * @since 1.7
     */
    public Builder(HttpTransport transport, JsonFactory jsonFactory,
        HttpRequestInitializer httpRequestInitializer) {
      super(
          transport,
          jsonFactory,
          DEFAULT_ROOT_URL,
          DEFAULT_SERVICE_PATH,
          httpRequestInitializer,
          false);
    }

    /** Builds a new instance of {@link Licensing}. */
    @Override
    public Licensing build() {
      return new Licensing(getTransport(),
          getHttpRequestInitializer(),
          getRootUrl(),
          getServicePath(),
          getObjectParser(),
          getGoogleClientRequestInitializer(),
          getApplicationName(),
          getSuppressPatternChecks());
    }

    @Override
    public Builder setRootUrl(String rootUrl) {
      return (Builder) super.setRootUrl(rootUrl);
    }

    @Override
    public Builder setServicePath(String servicePath) {
      return (Builder) super.setServicePath(servicePath);
    }

    @Override
    public Builder setHttpRequestInitializer(HttpRequestInitializer httpRequestInitializer) {
      return (Builder) super.setHttpRequestInitializer(httpRequestInitializer);
    }

    @Override
    public Builder setApplicationName(String applicationName) {
      return (Builder) super.setApplicationName(applicationName);
    }

    @Override
    public Builder setSuppressPatternChecks(boolean suppressPatternChecks) {
      return (Builder) super.setSuppressPatternChecks(suppressPatternChecks);
    }

    /**
     * Set the {@link LicensingRequestInitializer}.
     *
     * @since 1.12
     */
    public Builder setLicensingRequestInitializer(
        LicensingRequestInitializer licensingRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(licensingRequestInitializer);
    }

    @Override
    public Builder setGoogleClientRequestInitializer(
        GoogleClientRequestInitializer googleClientRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(googleClientRequestInitializer);
    }
  }
}
